@@ -9,6 +9,7 @@
 #include <poppler/Array.h>   // getLength(), add(), get()
 #include <poppler/Dict.h>    // add(), is(), lookup()
 #include <poppler/Catalog.h>
+#include "Iref.h"
 
 // (Few users should #include this header directly.  It is provided as a
 // separate header only for the benefit of callers of the discouraged
@@ -31,6 +32,9 @@ struct PDF::PDF_rep {
   Object  *catalog_obj ;
   Dict    *catalog     ;
   Catalog *catalog2    ;
+  Iref     info_iref   ;
+  Object  *info_obj    ;
+  Dict    *info        ;
 };
 
 #endif
