@@ -13,6 +13,12 @@
 
 using std::string;
 typedef std::set<string> set;
+const int magic = 0x9f05; // deprecated
+
+// Beware that Libpoppler's Dict::add() method does not copy the key-string you
+// feed it.  In an earlier Libpoppler, didn't it used to?  Maybe not; I can't
+// clearly remember.  In any case, letting the key-string go out of scope now
+// invalidates the Dict.  ---THB, March 2010---
 
 namespace {
 
@@ -43,7 +49,7 @@ string PDF::update_catalog(
   const Page_no::PS_page_numbering &nog
 ) {
 
-  PDF_rep *const rep = pdf.get_PDF_rep(0x9f05); // deprecated
+  PDF_rep *const rep = pdf.get_PDF_rep(magic);
 
   // To understand this code, refer to the Libpoppler headers and
   // to Adobe's PDF Reference 1.7, sect. 8.3.1.
@@ -59,21 +65,23 @@ string PDF::update_catalog(
   }
 
   Object dict_Roman_obj;
+  char s_Roman[] = "S";
   {
     dict_Roman_obj.initDict(static_cast<XRef*>(0));
     Dict *const dict_Roman = dict_Roman_obj.getDict();
     Object name_Roman;
     { char s[] = "r"; name_Roman.initName(s); }
-    { char s[] = "S"; dict_Roman->add( s, &name_Roman ); }
+    dict_Roman->add( s_Roman, &name_Roman );
   }
 
   Object dict_Arabic_obj;
+  char s_Arabic[] = "S";
   {
     dict_Arabic_obj.initDict(static_cast<XRef*>(0));
     Dict *const dict_Arabic = dict_Arabic_obj.getDict();
     Object name_Arabic;
     { char s[] = "D"; name_Arabic.initName(s); }
-    { char s[] = "S"; dict_Arabic->add( s, &name_Arabic ); }
+    dict_Arabic->add( s_Arabic, &name_Arabic );
   }
 
   Object array_obj;
@@ -91,17 +99,20 @@ string PDF::update_catalog(
   }
 
   Object dict_obj;
+  char s_Nums[] = "Nums";
   {
     dict_obj.initDict(static_cast<XRef*>(0));
     Dict *const dict = dict_obj.getDict();
-    { char s[] = "Nums"; dict->add( s, &array_obj ); }
+    dict->add( s_Nums, &array_obj );
   }
 
   Object ref_obj;
   ref_obj.initRef( n_obj(pdf), 0 );
 
-  { char s[] = "PageLabels"; catalog->add( s, &dict_obj ); }
-  { char s[] = "Outlines"  ; catalog->add( s, &ref_obj  ); }
+  char s_PageLabels[] = "PageLabels";
+  char s_Outlines  [] = "Outlines"  ;
+  catalog->add( s_PageLabels, &dict_obj );
+  catalog->add( s_Outlines  , &ref_obj  );
 
   string res;
   {
@@ -136,13 +147,75 @@ string PDF::update_catalog(
 
 }
 
+string PDF::add_title_to_info(
+  PDF &pdf,
+  const string &title
+) {
+
+  PDF_rep *const rep = pdf.get_PDF_rep(magic);
+
+  Object info_obj;
+  info_obj.initDict(static_cast<XRef*>(0));
+  Dict *info = info_obj.getDict();
+  {
+    set keys;
+    copy_but( info, rep->info, keys );
+  }
+
+  char s_Title[] = "Title";
+  {
+    Object obj_old;
+    info->lookup( s_Title, &obj_old );
+    if ( obj_old.isNull() ) {
+      Object obj_new;
+      {
+        GooString gs( title.c_str() );
+        obj_new.initString( &gs );
+      }
+      info->add( s_Title, &obj_new );
+    }
+  }
+
+  string res;
+  {
+    const Iref iref = iref_info(pdf);
+    std::ostringstream oss;
+    oss
+      << std::setw(TOC::width_i_obj) << iref.i
+      << " " << iref.gen << " obj\n";
+    res += oss.str();
+  }
+
+  // Do print() to a string rather than to stdout or a file.
+  {
+    int fd[2];
+    pipe(fd);
+    {
+      FILE *q = fdopen( fd[1], "w" );
+      info_obj.print(q);
+      fclose(q);
+    }
+    {
+      FILE *q = fdopen( fd[0], "r" );
+      int c;
+      while ( (c=fgetc(q)) != EOF ) res += c;
+      fclose(q);
+    }
+  }
+
+  res += "\nendobj\n";
+
+  return res;
+
+}
+
 string PDF::update_trailer(
   PDF &pdf,
   const int n_pdf_obj,
   const int offset_xref
 ) {
 
-  PDF_rep *const rep = pdf.get_PDF_rep(0x9f05); // deprecated
+  PDF_rep *const rep = pdf.get_PDF_rep(magic);
 
   // To understand this code, refer to the Libpoppler headers and
   // to Adobe's PDF Reference 1.7, sect. 3.4.4.
@@ -158,16 +231,18 @@ string PDF::update_trailer(
     copy_but( new_trailer, rep->trailer, keys );
   }
 
+  char s_Size[] = "Size";
   {
     Object obj;
     obj.initInt( n_pdf_obj );
-    { char s[] = "Size"; new_trailer->add( s, &obj ); }
+    new_trailer->add( s_Size, &obj );
   }
 
+  char s_Prev[] = "Prev";
   {
     Object obj;
     obj.initInt( offset_last_xref_table(pdf) );
-    { char s[] = "Prev"; new_trailer->add( s, &obj ); }
+    new_trailer->add( s_Prev, &obj );
   }
 
   string res = "trailer\n";
